//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 9 January 1998
// Last Modified: Thu Feb 12 18:25:33 GMT-0800 1998
// Filename:      ...sig/examples/sigmidi/improv/synthImprov/shadow2.cc
// Syntax:        C++; Synth Improv
//  
// Description: echos a note played on the synthesizer keyboard
//     Unlike shadow.cpp, this program also imitates the duration
//     of the echoing note and is polyphonic.

#include "synthImprov.h" 


/*----------------- beginning of improvization algorithms ---------------*/

int shadowDistance = 12;     // number of half-steps to place shadow at
int shadowSide     = 1;      // 1 = higher, -1 = lower
MidiMessage noteMessage;     // for reading keyno and velocity (and time)
int shadowNote = 0;          // note to play as a shadow
int keyray[128] = {0};       // keyboard array which keeps track of notes
int octave = 4;              // octave for computer keyboard notes


// user functions not needed by the synthImprov environment:
void interpretNote(MidiMessage message);



/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << endl;
   printboxtop();
   psl(" Shadow2 -- by Craig Stuart Sapp <craig@ccrma.stanford.edu>, 5 Jan 1998");
   psl("");
   psl(" Description:");
   psl("   This program will echo the input notes at a certain distance from");
   psl("   the original notes.  The range of the shadow is from 1 to 24");
   psl("   half-steps on either side of the original note.");
   psl("   Polyphonic version.");
   printintermediateline();
   psl("  User commands:");
   psl("     \"-\" = decrease shadow length   \"=\" = increase shadow length ");
   psl("     \"\\\" = switch shadow side             ");
   psl("");
   psl("  Computer synthesizer keyboard:");
   psl("      \"0\"-\"9\" = octave number of computer keyboard notes");
   psl("      Notes:           s   d      g    h   j   ");
   psl("                     z   x   c   v   b   n   m  ");
   printboxbottom();
   cout << endl;
} 


void initialization(void) { }


void finishup(void) { }



/*-------------------- main loop algorithms -----------------------------*/


void mainloopalgorithms(void) { 
   if (synth.getNoteCount() > 0) {
      interpretNote(synth.extractNote());
   }
}



//////////////////////////////
//
// interpretNote -- determine what to do with an incoming note (on or off).
//

void interpretNote(MidiMessage message) {
   int newNote = message.p1();
   int newVelocity = message.p2();

   if (newVelocity != 0) {
      // pick a shadow note
      shadowNote = newNote + shadowSide * shadowDistance;
      // ignore the shadow note if it is out of keyboard range:
      if (shadowNote > 0 && shadowNote < 128) {
         // turn off any old note (should never happen, but be safe):
         if (keyray[newNote] != 0) {
            synth.play(0, keyray[newNote], 0);
         }
         keyray[newNote] = shadowNote;
         synth.play(0, shadowNote, newVelocity);
      }
   } else {
      // a note off command
      if (keyray[newNote] != 0) {
         synth.play(0, keyray[newNote], 0);
         keyray[newNote] = 0;
      }
   }
}
      


/*-------------------- triggered algorithms -----------------------------*/

void keyboardchar(int key) { 
   if (isdigit((char)key)) {
      octave = key - '0';
      return;
   }
   switch (key) {
      case '-': 
         shadowDistance--; 
         if (shadowDistance < 1) shadowDistance = 1;
         cout << "Shadow = " << shadowSide * shadowDistance << "\t\t\t" << flush;
         break;
      case '=':
         shadowDistance++; 
         if (shadowDistance > 24) shadowDistance = 24;
         cout << "\r Shadow = " << shadowSide * shadowDistance << "\t\t\t" << flush;
         break;
      case '\\':
         shadowSide *= -1;
         cout << "\r Shadow = " << shadowSide * shadowDistance << "\t\t\t" << flush;
         break;
      default:
         octavekeyboard(key, octave);
   }
}



/*------------------ end improvization algorithms -----------------------*/



