//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed Jan 21 22:35:31 GMT-0800 1998
// Last Modified: Thu Jan 22 23:13:54 GMT-0800 1998
// Filename:      ...sig/code/control/MidiInPort/MidiInPort.h
// Syntax:        C++ 
//


#ifndef _MIDIINPORT_H_INCLUDED
#define _MIDIINPORT_H_INCLUDED

#define MIDIINPORT(system)   MidiInPort_ ## system

#include "MidiMessage.h"

#ifdef VISUAL
   #define sys    visual
   #include "MidiInPort_visual.h"
#else
   #define sys    unsupported
   #include "MidiInPort_unsupported.h"
#endif

// need to find out how to use define in MIDIINPORT

#ifdef VISUAL
class MidiInPort : protected MIDIINPORT(visual) {
   public:
                  MidiInPort         (void) : MIDIINPORT(visual)() {}
                  MidiInPort         (int aPort, int autoOpen = 1) : 
                                        MIDIINPORT(visual)(aPort, autoOpen) {}
                 ~MidiInPort()       { }

      void        close(void)        { MIDIINPORT(visual)::close(); }
      void        closeAll(void)     { MIDIINPORT(visual)::closeAll(); }
      MidiMessage extract(void)      { return MIDIINPORT(visual)::extract(); }
      int         getCount(void)     { return MIDIINPORT(visual)::getCount(); }
      const char* getName(void)      { return MIDIINPORT(visual)::getName(); }
      int         getNumPorts(void) { 
                     return MIDIINPORT(visual)::getNumPorts(); }
      int         getPort(void)      { return MIDIINPORT(visual)::getPort(); }
      int         getPortStatus(void){ 
                     return MIDIINPORT(visual)::getPortStatus(); }
      int         getTrace(void)     { return MIDIINPORT(visual)::getTrace(); }
      void        insert(const MidiMessage& aMessage) {
                     MIDIINPORT(visual)::insert(aMessage); }
      int         open(void)         { return MIDIINPORT(visual)::open(); }
      MidiMessage& operator[](int index) {
                     return MIDIINPORT(visual)::message(index); }
      void        pause(void)        { MIDIINPORT(visual)::pause(); }
      void        setBufferSize(int aSize) {
                     MIDIINPORT(visual)::setBufferSize(aSize); }
      void        setPort(int aPort) { MIDIINPORT(visual)::setPort(aPort); }
      int         setTrace(int aState) { 
                     return MIDIINPORT(visual)::setTrace(aState); }
      void        toggleTrace(void) { MIDIINPORT(visual)::toggleTrace(); }
      void        unpause(void)      { MIDIINPORT(visual)::unpause(); }
};
#else
class MidiInPort : protected MIDIINPORT(unsupported) {
   public:
                  MidiInPort         (void) : MIDIINPORT(unsupported)() {}
                  MidiInPort         (int aPort, int autoOpen = 1) : 
                                        MIDIINPORT(unsupported)(aPort, 
                                           autoOpen) {}
                 ~MidiInPort()       { }

      void        close(void)        { MIDIINPORT(unsupported)::close(); }
      void        closeAll(void)     { MIDIINPORT(unsupported)::closeAll(); }
      MidiMessage extract(void) { 
                     return MIDIINPORT(unsupported)::extract(); }
      int         getCount(void) { 
                     return MIDIINPORT(unsupported)::getCount(); }
      const char* getName(void) { 
                     return MIDIINPORT(unsupported)::getName(); }
      int         getNumPorts(void) { 
                     return MIDIINPORT(unsupported)::getNumPorts(); }
      int         getPort(void) { 
                     return MIDIINPORT(unsupported)::getPort(); }
      int         getPortStatus(void) { 
                     return MIDIINPORT(unsupported)::getPortStatus(); }
      int         getTrace(void) { 
                     return MIDIINPORT(unsupported)::getTrace(); }
      void        insert(const MidiMessage& aMessage) {
                     MIDIINPORT(unsupported)::insert(aMessage); }
      int         open(void)         { return MIDIINPORT(unsupported)::open(); }
      MidiMessage& operator[](int index) {
                     return MIDIINPORT(unsupported)::message(index); }
      void        pause(void)        { MIDIINPORT(unsupported)::pause(); }
      void        setBufferSize(int aSize) {
                     MIDIINPORT(unsupported)::setBufferSize(aSize); }
      void        setPort(int aPort) { MIDIINPORT(unsupported)::setPort(aPort);}
      int         setTrace(int aState) { 
                     return MIDIINPORT(unsupported)::setTrace(aState); }
      void        toggleTrace(void) { MIDIINPORT(unsupported)::toggleTrace(); }
      void        unpause(void)      { MIDIINPORT(unsupported)::unpause(); }
};
#endif



#endif  // _MIDIINPORT_H_INCLUDED



